//Il software è stato scritto da Daniel Berrangé, ed è liberamente scaricabile dal sito http://fstop138.berrange.com/2014/01/building-an-barn-door-mount-part-1-arduino-stepper-motor-control/ dove è spiegata tutta la procedura
//Io mi sono limitato a tradurre alcuni commenti e ad aggiungere il codice per la visualizzazione su LCD 16x2
//Prima di scaricare il software è necessario aggiungere le librerie sottostanti all'IDE


#include <core_build_options.h>
#include <secTimer.h>

// http://arduino-info.wikispaces.com/HAL-LibrariesUpdates
#include <FiniteStateMachine.h>

// http://www.airspayce.com/mikem/arduino/AccelStepper/
#include <AccelStepper.h>

//libreria gestione lcd 16x2
#include <LiquidCrystal.h>
// We don't want to send debug over the serial port by default since
// it seriously slows down the main loop causing tracking errors
//#define DEBUG
secTimer myTimer;   //variabile tempo di esposizione
unsigned long seconds=0; 
int TMax; //tempo massimo in secondi (da settare nel void setup


static const float STEP_SIZE_DEG = 1.8;  // Gradi per ogni step
static const float MICRO_STEPS = 8;      // Numero microstep
static const float THREADS_PER_CM = 8;   // Numero filetti per cm
static const float BASE_LEN_CM = 30.5;   // Lunghezza da cerniera a centro barra filettata
static const float INITIAL_ANGLE = 0;    // Angolo iniziale all'accensione
static const float MAXIMUM_ANGLE = 30;   // Angolo massimo di apertura (30°= 2h)

//Anche cambiando il circuito niente del codice sopra va modificato

// Collegamenti con arduino
static const int pinOutStep = 9;      // STEP a4988
static const int pinOutDirection = 8; // DIR a4988

static const int pinInSidereal = 4;  // Arduino analogue pin connected to sidereal mode switch   
static const int pinInHighspeed = 5; // Arduino analogue pin connected to highspeed mode switch
static const int pinInDirection = 3; // Arduino analogue pin connected to direction switch
LiquidCrystal lcd(12, 11, 5, 4, 3, 2); //impostazione pin lcd



static const float USTEPS_PER_ROTATION = 360.0 / STEP_SIZE_DEG * MICRO_STEPS; // usteps per rotazione barra


static const float SIDE_REAL_SECS = 86164.0419; // tempo siderale


static AccelStepper motor(AccelStepper::DRIVER,
                          pinOutStep,
                          pinOutDirection);

// Stati macchina: siderale, accelerato e spento
static State stateSidereal = State(state_sidereal_enter, state_sidereal_update, state_sidereal_exit);
static State stateHighspeed = State(state_highspeed_enter, state_highspeed_update, state_highspeed_update);
static State stateOff = State(state_off_enter, state_off_update, state_off_exit);
static FSM barndoor = FSM(stateOff);


// Numero step per chiudere completamente l'inseguitore
long time_to_usteps(long tsecs)
{
    return (long)(USTEPS_PER_ROTATION *
                  THREADS_PER_CM * 2.0 * BASE_LEN_CM *
                  sin(tsecs * PI / SIDE_REAL_SECS));
}


// Numero di step per fare un determinato angolo
long angle_to_usteps(float angle)
{
    return time_to_usteps(SIDE_REAL_SECS / 360.0 * angle);
}


// Corrispondenza numero di passi in tempo di inseguimento
long usteps_to_time(long usteps)
{
    return (long)(asin(usteps /
                       (USTEPS_PER_ROTATION * THREADS_PER_CM * 2.0 * BASE_LEN_CM)) *
                  SIDE_REAL_SECS / PI);
}

// Condizioni iniziali all'accensione del motore

// Offset iniziale segnato in INITIAL_ANGLE
static long offsetPositionUSteps;
// Angolo di apertura massima
static long maximumPositionUSteps;
// Passi motore fino allo start
static long startPositionUSteps;
// Totale tempo di inseguimento
static long startPositionSecs;
// Tempo da quando passa da fermo a avviato
static long startWallClockSecs;


// Variabili usante nel calcolo nel tempo

// Tempo quando si deve calcolare il prossimo passo
static long targetWallClockSecs;
// Tempo di inseguimento
static long targetPositionSecs;
// STEP motore
static long targetPositionUSteps;

void setup(void)
{
  lcd.begin(16, 2);  //tipo di lcd (16 colonne, 2 righe)
  lcd.print("Astroinseguitore");  //scrive astroinseguitore sulla prima riga
    pinMode(pinInSidereal, OUTPUT);
    pinMode(pinInHighspeed, OUTPUT);
    pinMode(pinInDirection, OUTPUT);
    TMax=7000; //tempo massimo
  
    motor.setPinsInverted(true, false, false);
    motor.setMaxSpeed(3000);

    offsetPositionUSteps = angle_to_usteps(INITIAL_ANGLE);
    maximumPositionUSteps = angle_to_usteps(MAXIMUM_ANGLE);

#ifdef DEBUG
    Serial.begin(9600);
#endif
}


// Tiene conto dell'angolo iniziale
long motor_position()
{
    return motor.currentPosition() + offsetPositionUSteps;
}


// Viene chiamata quando si attiva il motore per determinare la sua posizione
void start_tracking(void)
{
    startPositionUSteps = motor_position();
    startPositionSecs = usteps_to_time(startPositionUSteps);
    startWallClockSecs = millis() / 1000;
    targetWallClockSecs = startWallClockSecs;

#ifdef DEBUG
    Serial.print("Enter sidereal\n");
    Serial.print("offset pos usteps: ");
    Serial.print(offsetPositionUSteps);
    Serial.print(", start pos usteps: ");
    Serial.print(startPositionUSteps);
    Serial.print(", start pos secs: ");
    Serial.print(startPositionSecs);
    Serial.print(", start wclk secs: ");
    Serial.print(startWallClockSecs);
    Serial.print("\n\n");
#endif
}


// Chiama la funzione quando deve calcolare una nuova posizione
void plan_tracking(void)
{
    targetWallClockSecs = targetWallClockSecs + 15;
    targetPositionSecs = startPositionSecs + (targetWallClockSecs - startWallClockSecs);
    targetPositionUSteps = time_to_usteps(targetPositionSecs);

#ifdef DEBUG
    Serial.print("target pos usteps: ");
    Serial.print(targetPositionUSteps);
    Serial.print(", target pos secs: ");
    Serial.print(targetPositionSecs);
    Serial.print(", target wclk secs: ");
    Serial.print(targetWallClockSecs);
    Serial.print("\n");
#endif
}


// Chiamato ad ogni passo, applica le correzioni automatiche
void apply_tracking(long currentWallClockSecs)
{
    long timeLeft = targetWallClockSecs - currentWallClockSecs;
    long stepsLeft = targetPositionUSteps - motor_position();
    float stepsPerSec = (float)stepsLeft / (float)timeLeft;

#ifdef DEBUG32
    Serial.print("Target ");
    Serial.print(targetPositionUSteps);
    Serial.print("  curr ");
    Serial.print(motor_position());
    Serial.print("  left");
    Serial.print(stepsLeft);
    Serial.print("\n");
#endif

    if (motor_position() >= maximumPositionUSteps) {
        motor.stop();
    } else {
        motor.setSpeed(stepsPerSec);
        motor.runSpeed();
    }
}


// Chiamato durante il passaggio da fermo a siderale
void state_sidereal_enter(void)
{
    start_tracking();
    plan_tracking();
}


// chiamato durante il movimento siderale
void state_sidereal_update(void)
{
    long currentWallClockSecs = millis() / 1000;

    if (currentWallClockSecs >= targetWallClockSecs) {
        plan_tracking();
    }

    apply_tracking(currentWallClockSecs);
}

void state_sidereal_exit(void)
{
    // nada
}

void state_highspeed_enter(void)
{
#ifdef DEBUG
    Serial.print("Enter highspeed\n");
#endif
}


// Chiamato ad ogni iterazione per evitare di mandare il motore sotto sforzo
void state_highspeed_update(void)
{
    // scelta direzione del motore 
    if (analogRead(pinInDirection) < 512) {
        if (motor_position() >= maximumPositionUSteps) {
            motor.stop();
        } else {
            motor.setSpeed(5000);
            motor.runSpeed();
        }
    } else {
        if (motor.currentPosition() <= 0) {
            motor.stop();
        } else {
            motor.setSpeed(-5000);
            motor.runSpeed();
        }
    }
}

void state_highspeed_exit(void)
{
    // nada
}

void state_off_enter(void)
{
#ifdef DEBUG
    Serial.print("Enter off\n");
#endif
    motor.stop();
}

void state_off_update(void)
{
    // nada
}

void state_off_exit(void)
{
    // nada
}


void loop(void)
{
    // stabilire la velocita: siderale, alta velocità e fermo 
        
    if (analogRead(pinInSidereal) < 512) {
        barndoor.transitionTo(stateSidereal);
         myTimer.startTimer(); //attiva il timer quando mov. siderale
                 seconds=myTimer.readTimer();
        lcd.print("Mov. Sid.");
    } else if (analogRead(pinInHighspeed) < 512) {
        barndoor.transitionTo(stateHighspeed);
        lcd.print("Mov. acc.");
    } else {
        barndoor.transitionTo(stateOff);
        lcd.print("Stop");
    }
    barndoor.update();
    
   
    lcd.setCursor(0, 1); //scrive sulla seconda riga a partire dalla prima colonna
    //ogni secondo aggiorna i minuti
    if(seconds!=myTimer.readTimer()) {
        seconds=myTimer.readTimer();
        lcd.setCursor(10, 1); 
        lcd.print(myTimer.readTimer());
    }
    //controllo tempo esposizione
    if (seconds==TMax) {
      barndoor.transitionTo(stateOff);
      lcd.print("Fine corsa");
    }
}
//Fine
